import 'package:equatable/equatable.dart';

enum Brand { bmw, mini }
enum Region { northAmerica, restOfWorld, korea }
enum Environment { daily, development, integration, production }
enum Mode { mock, real }
enum HostSystem { ios, android }

class StartupConfiguration extends Equatable {
  final Brand brand;
  final Region region;
  final Environment environment;
  final HostSystem hostSystem;
  final Mode mode;

  const StartupConfiguration(
    this.brand,
    this.region,
    this.environment,
    this.hostSystem, [
    this.mode = Mode.real,
  ]) : assert(hostSystem != null);

  @override
  List<Object> get props => [brand, region, environment, hostSystem, mode];
}

class ConnectedStartupConfiguration extends StartupConfiguration {
  final Region region;
  final Environment environment;
  final Brand brand;
  final HostSystem hostSystem;
  final Mode mode;

  ConnectedStartupConfiguration(
    this.brand,
    this.region,
    this.environment,
    this.hostSystem, [
    this.mode = Mode.real,
  ]) : super(brand, region, environment, hostSystem, mode);

  factory ConnectedStartupConfiguration.bmwNorthAmericaDailyMock(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.northAmerica,
        Environment.daily,
        hostSystem,
        Mode.mock,
      );

  factory ConnectedStartupConfiguration.bmwRestOfWorldDailyMock(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.restOfWorld,
        Environment.daily,
        hostSystem,
        Mode.mock,
      );

  factory ConnectedStartupConfiguration.bmwKoreaDailyMock(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.korea,
        Environment.daily,
        hostSystem,
        Mode.mock,
      );

  factory ConnectedStartupConfiguration.bmwNorthAmericaDaily(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.northAmerica,
        Environment.daily,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwRestOfWorldDaily(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.restOfWorld,
        Environment.daily,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwKoreaDaily(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.korea,
        Environment.daily,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwNorthAmericaDevelopment(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.northAmerica,
        Environment.development,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwRestOfWorldDevelopment(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.restOfWorld,
        Environment.development,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwKoreaDevelopment(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.korea,
        Environment.development,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwNorthAmericaIntegration(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.northAmerica,
        Environment.integration,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwRestOfWorldIntegration(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.restOfWorld,
        Environment.integration,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwKoreaIntegration(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.korea,
        Environment.integration,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwNorthAmericaProduction(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.northAmerica,
        Environment.production,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwRestOfWorldProduction(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.restOfWorld,
        Environment.production,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.bmwKoreaProduction(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.bmw,
        Region.korea,
        Environment.production,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniNorthAmericaDailyMock(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.northAmerica,
        Environment.daily,
        hostSystem,
        Mode.mock,
      );

  factory ConnectedStartupConfiguration.miniRestOfWorldDailyMock(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.restOfWorld,
        Environment.daily,
        hostSystem,
        Mode.mock,
      );

  factory ConnectedStartupConfiguration.miniKoreaDailyMock(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.korea,
        Environment.daily,
        hostSystem,
        Mode.mock,
      );

  factory ConnectedStartupConfiguration.miniNorthAmericaDaily(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.northAmerica,
        Environment.daily,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniRestOfWorldDaily(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.restOfWorld,
        Environment.daily,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniKoreaDaily(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.korea,
        Environment.daily,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniNorthAmericaDevelopment(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.northAmerica,
        Environment.development,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniRestOfWorldDevelopment(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.restOfWorld,
        Environment.development,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniKoreaDevelopment(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.korea,
        Environment.development,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniNorthAmericaIntegration(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.northAmerica,
        Environment.integration,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniRestOfWorldIntegration(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.restOfWorld,
        Environment.integration,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniKoreaIntegration(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.korea,
        Environment.integration,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniNorthAmericaProduction(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.northAmerica,
        Environment.production,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniRestOfWorldProduction(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.restOfWorld,
        Environment.production,
        hostSystem,
      );

  factory ConnectedStartupConfiguration.miniKoreaProduction(
    HostSystem hostSystem,
  ) =>
      ConnectedStartupConfiguration(
        Brand.mini,
        Region.korea,
        Environment.production,
        hostSystem,
      );
}
