import 'package:test/test.dart';

import 'package:startup_configuration/startup_configuration.dart';

void main() {
  group('Props', () {
    test('are correct', () {
      expect(
        StartupConfiguration(
          Brand.bmw,
          Region.northAmerica,
          Environment.daily,
          HostSystem.ios,
          Mode.real,
        ).props,
        [
          Brand.bmw,
          Region.northAmerica,
          Environment.daily,
          HostSystem.ios,
          Mode.real,
        ],
      );
    });
  });

  group('HostSystem', () {
    test('only iOS and Android are supported', () {
      expect(HostSystem.values.length, 2);
      expect(HostSystem.values.contains(HostSystem.android), isTrue);
      expect(HostSystem.values.contains(HostSystem.ios), isTrue);
    });

    test('cannot create a configuration with a null a host system', () {
      try {
        ConnectedStartupConfiguration.bmwNorthAmericaDailyMock(null);
      } catch (error) {
        expect(error is AssertionError, true);
      }
    });
  });

  group('BMW', () {
    group('Daily', () {
      test(
          'bmwNorthAmericaDailyMock configuration has brand: bmw, environment: daily, region: northAmerica, and mode mock',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwNorthAmericaDailyMock(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);
        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.mock);
      });

      test(
          'bmwNorthAmericaDaily configuration has brand: bmw, environment: daily, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwNorthAmericaDaily(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);
        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.real);
      });
      test(
          'bmwRestOfWorldDailyMock configuration has brand: bmw, environment: daily, region: restOfWorld, and mode mock',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwRestOfWorldDailyMock(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);
        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.mock);
      });
      test(
          'bmwRestOfWorldDaily configuration has brand: bmw, environment: daily and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwRestOfWorldDaily(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.real);
      });
      test(
          'bmwKoreaDailyMock configuration has brand: bmw, environment: daily, region: korea, and mode mock',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwKoreaDailyMock(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);
        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.mock);
      });
      test(
          'bmwKoreaDaily configuration has brand: bmw, environment: daily, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwKoreaDaily(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.real);
      });
    });

    group('Development', () {
      test(
          'bmwNorthAmericaDevelopment configuration has brand: bmw, environment: development, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwNorthAmericaDevelopment(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.development);
        expect(configuration.mode, Mode.real);
      });
      test(
          'bmwRestOfWorldDevelopment configuration has brand: bmw, environment: development, and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwRestOfWorldDevelopment(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.development);
        expect(configuration.mode, Mode.real);
      });
      test(
          'bmwKoreaDevelopment configuration has brand: bmw, environment: development, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwKoreaDevelopment(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.development);
        expect(configuration.mode, Mode.real);
      });
    });

    group('Integration', () {
      test(
          'bmwNorthAmericaIntegration configuration has brand: bmw, environment: integration, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwNorthAmericaIntegration(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.integration);
        expect(configuration.mode, Mode.real);
      });
      test(
          'bmwRestOfWorldIntegration configuration has brand: bmw, environment: integration, and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwRestOfWorldIntegration(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.integration);
        expect(configuration.mode, Mode.real);
      });
      test(
          'bmwKoreaIntegration configuration has brand: bmw, environment: integration, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwKoreaIntegration(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.integration);
        expect(configuration.mode, Mode.real);
      });
    });

    group('Production', () {
      test(
          'bmwNorthAmericaProduction configuration has brand: bmw, environment: production, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwNorthAmericaProduction(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.production);
        expect(configuration.mode, Mode.real);
      });

      test(
          'bmwRestOfWorldProduction configuration has brand: bmw, environment: production, and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.bmwRestOfWorldProduction(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.production);
        expect(configuration.mode, Mode.real);
      });

      test(
          'bmwKoreaProduction configuration has brand: bmw, environment: production, and region: korea',
          () {
        final configuration = ConnectedStartupConfiguration.bmwKoreaProduction(
            HostSystem.android);
        expect(configuration.hostSystem, HostSystem.android);

        expect(configuration.brand, Brand.bmw);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.production);
        expect(configuration.mode, Mode.real);
      });
    });
  });

  group('MINI', () {
    group('Daily', () {
      test(
          'miniNorthAmericaDailyMock configuration has brand: mini, environment: daily, region: northAmerica, and mode mock',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniNorthAmericaDailyMock(
                HostSystem.android);
        expect(configuration.hostSystem, HostSystem.android);
        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.mock);
      });
      test(
          'miniNorthAmericaDaily configuration has brand: mini, environment: daily, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniNorthAmericaDaily(
                HostSystem.android);
        expect(configuration.hostSystem, HostSystem.android);
        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.real);
      });
      test(
          'miniRestofWorldDailyMock configuration has brand: mini, environment: daily, region: restOfWorld, and mode mock',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniRestOfWorldDailyMock(
                HostSystem.android);
        expect(configuration.hostSystem, HostSystem.android);
        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.mock);
      });
      test(
          'miniRestOfWorldDaily configuration has brand: mini, environment: daily and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniRestOfWorldDaily(
                HostSystem.android);
        expect(configuration.hostSystem, HostSystem.android);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.real);
      });
      test(
          'miniKoreaDailyMock configuration has brand: mini, environment: daily, region: korea, and mode mock',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniKoreaDailyMock(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);
        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.mock);
      });
      test(
          'miniKoreaDaily configuration has brand: mini, environment: daily, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniKoreaDaily(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.daily);
        expect(configuration.mode, Mode.real);
      });
    });

    group('Development', () {
      test(
          'miniNorthAmericaDevelopment configuration has brand: mini, environment: development, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniNorthAmericaDevelopment(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.development);
        expect(configuration.mode, Mode.real);
      });
      test(
          'miniRestOfWorldDevelopment configuration has brand: mini, environment: development, and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniRestOfWorldDevelopment(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.development);
        expect(configuration.mode, Mode.real);
      });
      test(
          'miniKoreaDevelopment configuration has brand: mini, environment: development, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniKoreaDevelopment(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.development);
        expect(configuration.mode, Mode.real);
      });
    });

    group('Integration', () {
      test(
          'miniNorthAmericaIntegration configuration has brand: mini, environment: integration, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniNorthAmericaIntegration(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.integration);
        expect(configuration.mode, Mode.real);
      });
      test(
          'miniRestOfWorldIntegration configuration has brand: mini, environment: integration, and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniRestOfWorldIntegration(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.integration);
        expect(configuration.mode, Mode.real);
      });
      test(
          'miniKoreaIntegration configuration has brand: mini, environment: integration, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniKoreaIntegration(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.integration);
        expect(configuration.mode, Mode.real);
      });
    });

    group('Production', () {
      test(
          'miniNorthAmericaProduction configuration has brand: mini, environment: production, and region: northAmerica',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniNorthAmericaProduction(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.northAmerica);
        expect(configuration.environment, Environment.production);
        expect(configuration.mode, Mode.real);
      });

      test(
          'miniRestOfWorldProduction configuration has brand: mini, environment: production, and region: restOfWorld',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniRestOfWorldProduction(
                HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.restOfWorld);
        expect(configuration.environment, Environment.production);
        expect(configuration.mode, Mode.real);
      });

      test(
          'miniKoreaProduction configuration has brand: mini, environment: production, and region: korea',
          () {
        final configuration =
            ConnectedStartupConfiguration.miniKoreaProduction(HostSystem.ios);
        expect(configuration.hostSystem, HostSystem.ios);

        expect(configuration.brand, Brand.mini);
        expect(configuration.region, Region.korea);
        expect(configuration.environment, Environment.production);
        expect(configuration.mode, Mode.real);
      });
    });
  });
}
